@@ -545,7 +545,6 @@ class Enforcer(object):
         self.use_conf = use_conf
         self._need_check_rule = True
         self.overwrite = overwrite
-        self._loaded_files = []
         self._policy_dir_mtimes = {}
         self._file_cache = {}
         self._informed_no_policy_file = False
@@ -586,7 +585,6 @@ class Enforcer(object):
         self.set_rules({})
         self.default_rule = None
         self.policy_path = None
-        self._loaded_files = []
         self._policy_dir_mtimes = {}
         self._file_cache.clear()
         self.registered_rules = {}
@@ -627,22 +625,48 @@ class Enforcer(object):
                     overwrite=self.overwrite
                 )
 
-            force_reload_policy_dir = force_reload
+            force_reload_policy_dirs = force_reload
             if policy_file_rules_changed:
-                force_reload_policy_dir = True
+                force_reload_policy_dirs = True
+            existing_policy_dirs = []
 
             for path in self.conf.oslo_policy.policy_dirs:
                 try:
-                    path = self._get_policy_path(path)
+                    absolute_path = self._get_policy_path(path)
+                    existing_policy_dirs.append(absolute_path)
                 except cfg.ConfigFilesNotFoundError:
                     continue
-                if (self._is_directory_updated(self._policy_dir_mtimes, path)
-                        or force_reload_policy_dir):
+                # If change was made in any policy directory or main policy
+                # file then all policy directories and main file are
+                # re-calculated from scratch. We don't have separate rule sets
+                # for every policy folder, we only have the only rule set in
+                # RAM for all rule configs (self.rules). So it's the only way
+                # to be consistent.
+                if self._is_directory_updated(self._policy_dir_mtimes,
+                                              absolute_path):
+                    force_reload_policy_dirs = True
+            if force_reload_policy_dirs and existing_policy_dirs:
+                # Here we realize that some policy folders or main policy file
+                # were changed and we need to recalculate all rules from
+                # scratch.
+                # If policy_file_rules_changed is True then we know:
+                # 1. all rules were already reset.
+                # 2. rules from main policy file were already applied.
+                # Otherwise main policy file was not changed and rules were not
+                # reset and. So we reset rules and force to re-calculate
+                # rules in main policy file. And after that we apply rules
+                # from every policy directory.
+                if self.policy_path:
+                    if not policy_file_rules_changed and self.overwrite:
+                        self._load_policy_file(path=self.policy_path,
+                                               force_reload=True,
+                                               overwrite=self.overwrite
+                                               )
+                elif self.overwrite:
+                    self.rules = Rules(default_rule=self.default_rule)
+                for path in existing_policy_dirs:
                     self._walk_through_policy_directory(
-                        path,
-                        self._load_policy_file,
-                        force_reload_policy_dir, False
-                    )
+                        path, self._load_policy_file, True, False)
 
             for default in self.registered_rules.values():
                 if default.deprecated_for_removal:
@@ -917,7 +941,6 @@ class Enforcer(object):
             self.set_rules(rules, overwrite=overwrite, use_conf=True)
             rules_changed = True
             self._record_file_rules(data, overwrite)
-            self._loaded_files.append(path)
             LOG.debug('Reloaded policy file: %(path)s', {'path': path})
         return rules_changed
 
@@ -982,6 +1005,17 @@ class Enforcer(object):
             )
             raise InvalidContextObject(msg)
 
+        # NOTE(lbragstad): We unfortunately have to special case this
+        # attribute. Originally when the system scope when into oslo.policy, we
+        # checked for a key called 'system' in creds. The oslo.context library
+        # uses `system_scope` instead, and the compatibility between
+        # oslo.policy and oslo.context was an afterthought. We'll have to
+        # support services who've been setting creds['system'], but we can do
+        # that by making sure we populate it with what's in the context object
+        # if it has a system_scope attribute.
+        if creds.get('system_scope'):
+            creds['system'] = creds.get('system_scope')
+
         if LOG.isEnabledFor(logging.DEBUG):
             try:
                 creds_dict = strutils.mask_dict_password(creds)
@@ -1007,6 +1041,8 @@ class Enforcer(object):
         if isinstance(rule, _checks.BaseCheck):
             # If the thing we're given is a Check, we don't know the
             # name of the rule, so pass None for current_rule.
+            if rule.scope_types:
+                self._enforce_scope(creds, rule)
             result = _checks._check(
                 rule=rule,
                 target=target,
@@ -1029,40 +1065,9 @@ class Enforcer(object):
                 #                 as token_scope is not actually a hardcoded
                 #                 token.
 
-                # Check the scope of the operation against the possible scope
-                # attributes provided in `creds`.
-                if creds.get('system'):
-                    token_scope = 'system'  # nosec
-                elif creds.get('domain_id'):
-                    token_scope = 'domain'  # nosec
-                else:
-                    # If the token isn't system-scoped or domain-scoped then
-                    # we're dealing with a project-scoped token.
-                    token_scope = 'project'  # nosec
-
                 registered_rule = self.registered_rules.get(rule)
                 if registered_rule and registered_rule.scope_types:
-                    if token_scope not in registered_rule.scope_types:
-                        if self.conf.oslo_policy.enforce_scope:
-                            raise InvalidScope(
-                                rule, registered_rule.scope_types, token_scope
-                            )
-                        # If we don't raise an exception we should at least
-                        # inform operators about policies that are being used
-                        # with improper scopes.
-                        msg = (
-                            'Policy %(rule)s failed scope check. The token '
-                            'used to make the request was %(token_scope)s '
-                            'scoped but the policy requires %(policy_scope)s '
-                            'scope. This behavior may change in the future '
-                            'where using the intended scope is required' % {
-                                'rule': rule,
-                                'token_scope': token_scope,
-                                'policy_scope': registered_rule.scope_types
-                            }
-                        )
-                        warnings.warn(msg)
-
+                    self._enforce_scope(creds, registered_rule)
                 result = _checks._check(
                     rule=to_check,
                     target=target,
@@ -1080,6 +1085,39 @@ class Enforcer(object):
 
         return result
 
+    def _enforce_scope(self, creds, rule):
+        # Check the scope of the operation against the possible scope
+        # attributes provided in `creds`.
+        if creds.get('system'):
+            token_scope = 'system'  # nosec
+        elif creds.get('domain_id'):
+            token_scope = 'domain'  # nosec
+        else:
+            # If the token isn't system-scoped or domain-scoped then
+            # we're dealing with a project-scoped token.
+            token_scope = 'project'  # nosec
+
+        if token_scope not in rule.scope_types:
+            if self.conf.oslo_policy.enforce_scope:
+                raise InvalidScope(
+                    rule, rule.scope_types, token_scope
+                )
+            # If we don't raise an exception we should at least
+            # inform operators about policies that are being used
+            # with improper scopes.
+            msg = (
+                'Policy %(rule)s failed scope check. The token '
+                'used to make the request was %(token_scope)s '
+                'scoped but the policy requires %(policy_scope)s '
+                'scope. This behavior may change in the future '
+                'where using the intended scope is required' % {
+                    'rule': rule,
+                    'token_scope': token_scope,
+                    'policy_scope': rule.scope_types
+                }
+            )
+            warnings.warn(msg)
+
     def _map_context_attributes_into_creds(self, context):
         creds = {}
         # port public context attributes into the creds dictionary so long as
@@ -1088,17 +1126,6 @@ class Enforcer(object):
         for k, v in context_values.items():
             creds[k] = v
 
-        # NOTE(lbragstad): We unfortunately have to special case this
-        # attribute. Originally when the system scope when into oslo.policy, we
-        # checked for a key called 'system' in creds. The oslo.context library
-        # uses `system_scope` instead, and the compatibility between
-        # oslo.policy and oslo.context was an afterthought. We'll have to
-        # support services who've been setting creds['system'], but we can do
-        # that by making sure we populate it with what's in the context object
-        # if it has a system_scope attribute.
-        if context.system_scope:
-            creds['system'] = context.system_scope
-
         return creds
 
     def register_default(self, default):
