@@ -98,10 +98,8 @@ class GenerateSampleYAMLTestCase(base.PolicyBaseTestCase):
 # POST  /test/
 #"owner": "project_id:%(project_id)s"
 
-#
 #"shared": "field:networks:shared=True"
 
-#
 #"admin_or_owner": "rule:admin or rule:owner"
 
 '''
@@ -137,10 +135,8 @@ class GenerateSampleYAMLTestCase(base.PolicyBaseTestCase):
 # POST  /test/
 #"owner": "project_id:%(project_id)s"
 
-#
 #"shared": "field:networks:shared=True"
 
-#
 #"admin_or_owner": "rule:admin or rule:owner"
 
 '''
@@ -198,17 +194,17 @@ class GenerateSampleYAMLTestCase(base.PolicyBaseTestCase):
     def test_deprecated_policies_are_aliased_to_new_names(self):
         deprecated_rule = policy.DeprecatedRule(
             name='foo:post_bar',
-            check_str='role:fizz'
+            check_str='role:fizz',
+            deprecated_reason=(
+                'foo:post_bar is being removed in favor of foo:create_bar'
+            ),
+            deprecated_since='N',
         )
         new_rule = policy.RuleDefault(
             name='foo:create_bar',
             check_str='role:fizz',
             description='Create a bar.',
             deprecated_rule=deprecated_rule,
-            deprecated_reason=(
-                'foo:post_bar is being removed in favor of foo:create_bar'
-            ),
-            deprecated_since='N'
         )
         opts = {'rules': [new_rule]}
 
@@ -223,10 +219,12 @@ class GenerateSampleYAMLTestCase(base.PolicyBaseTestCase):
         expected = '''# Create a bar.
 #"foo:create_bar": "role:fizz"
 
-# DEPRECATED "foo:post_bar":"role:fizz" has been deprecated since N in
-# favor of "foo:create_bar":"role:fizz". foo:post_bar is being removed
-# in favor of foo:create_bar
+# DEPRECATED
+# "foo:post_bar":"role:fizz" has been deprecated since N in favor of
+# "foo:create_bar":"role:fizz".
+# foo:post_bar is being removed in favor of foo:create_bar
 "foo:post_bar": "rule:foo:create_bar"
+
 '''
         stdout = self._capture_stdout()
         with mock.patch('stevedore.named.NamedExtensionManager',
@@ -242,17 +240,17 @@ class GenerateSampleYAMLTestCase(base.PolicyBaseTestCase):
     def test_deprecated_policies_with_same_name(self):
         deprecated_rule = policy.DeprecatedRule(
             name='foo:create_bar',
-            check_str='role:old'
+            check_str='role:old',
+            deprecated_reason=(
+                'role:fizz is a more sane default for foo:create_bar'
+            ),
+            deprecated_since='N',
         )
         new_rule = policy.RuleDefault(
             name='foo:create_bar',
             check_str='role:fizz',
             description='Create a bar.',
             deprecated_rule=deprecated_rule,
-            deprecated_reason=(
-                'role:fizz is a more sane default for foo:create_bar'
-            ),
-            deprecated_since='N'
         )
         opts = {'rules': [new_rule]}
 
@@ -267,9 +265,11 @@ class GenerateSampleYAMLTestCase(base.PolicyBaseTestCase):
         expected = '''# Create a bar.
 #"foo:create_bar": "role:fizz"
 
-# DEPRECATED "foo:create_bar":"role:old" has been deprecated since N
-# in favor of "foo:create_bar":"role:fizz". role:fizz is a more sane
-# default for foo:create_bar
+# DEPRECATED
+# "foo:create_bar":"role:old" has been deprecated since N in favor of
+# "foo:create_bar":"role:fizz".
+# role:fizz is a more sane default for foo:create_bar
+
 '''
         stdout = self._capture_stdout()
         with mock.patch('stevedore.named.NamedExtensionManager',
@@ -474,6 +474,23 @@ class GenerateSampleJSONTestCase(base.PolicyBaseTestCase):
 
         self.assertEqual(expected, stdout.getvalue())
 
+    @mock.patch.object(generator, 'LOG')
+    def test_generate_json_file_log_warning(self, mock_log):
+        extensions = []
+        for name, opts in OPTS.items():
+            ext = stevedore.extension.Extension(name=name, entry_point=None,
+                                                plugin=None, obj=opts)
+            extensions.append(ext)
+        test_mgr = stevedore.named.NamedExtensionManager.make_test_instance(
+            extensions=extensions, namespace=['base_rules', 'rules'])
+
+        output_file = self.get_config_file_fullname('policy.json')
+        with mock.patch('stevedore.named.NamedExtensionManager',
+                        return_value=test_mgr):
+            generator._generate_sample(['base_rules', 'rules'], output_file,
+                                       output_format='json')
+            mock_log.warning.assert_any_call(policy.WARN_JSON)
+
 
 class GeneratorRaiseErrorTestCase(testtools.TestCase):
     def test_generator_raises_error(self):
@@ -492,8 +509,8 @@ class GeneratorRaiseErrorTestCase(testtools.TestCase):
                 raise FakeException()
 
         fake_ep = FakeEP()
-        fake_eps = mock.Mock(return_value=[fake_ep])
-        with mock.patch('pkg_resources.iter_entry_points', fake_eps):
+        with mock.patch('stevedore.named.NamedExtensionManager',
+                        side_effect=FakeException()):
             self.assertRaises(FakeException, generator._generate_sample,
                               fake_ep.name)
 
@@ -589,12 +606,18 @@ class ListRedundantTestCase(base.PolicyBaseTestCase):
         enforcer.register_default(
             policy.RuleDefault('owner', 'project_id:%(project_id)s'))
         # register a new opt
-        deprecated_rule = policy.DeprecatedRule('old_foo', 'role:bar')
+        deprecated_rule = policy.DeprecatedRule(
+            name='old_foo',
+            check_str='role:bar',
+            deprecated_reason='reason',
+            deprecated_since='T'
+        )
         enforcer.register_default(
-            policy.RuleDefault('foo', 'role:foo',
-                               deprecated_rule=deprecated_rule,
-                               deprecated_reason='reason',
-                               deprecated_since='T')
+            policy.RuleDefault(
+                name='foo',
+                check_str='role:foo',
+                deprecated_rule=deprecated_rule,
+            ),
         )
 
         # Mock out stevedore to return the configured enforcer
@@ -639,7 +662,9 @@ class UpgradePolicyTestCase(base.PolicyBaseTestCase):
         self.create_config_file('policy.json', policy_json_contents)
         deprecated_policy = policy.DeprecatedRule(
             name='deprecated_name',
-            check_str='rule:admin'
+            check_str='rule:admin',
+            deprecated_reason='test',
+            deprecated_since='Stein',
         )
         self.new_policy = policy.DocumentedRuleDefault(
             name='new_policy_name',
@@ -647,8 +672,6 @@ class UpgradePolicyTestCase(base.PolicyBaseTestCase):
             description='test_policy',
             operations=[{'path': '/test', 'method': 'GET'}],
             deprecated_rule=deprecated_policy,
-            deprecated_reason='test',
-            deprecated_since='Stein'
         )
         self.extensions = []
         ext = stevedore.extension.Extension(name='test_upgrade',
@@ -679,6 +702,23 @@ class UpgradePolicyTestCase(base.PolicyBaseTestCase):
                 self.assertIsNotNone(new_policy.get('new_policy_name'))
                 self.assertIsNone(new_policy.get('deprecated_name'))
 
+    @mock.patch.object(generator, 'LOG')
+    def test_upgrade_policy_json_file_log_warning(self, mock_log):
+        test_mgr = stevedore.named.NamedExtensionManager.make_test_instance(
+            extensions=self.extensions, namespace='test_upgrade')
+        with mock.patch('stevedore.named.NamedExtensionManager',
+                        return_value=test_mgr):
+            testargs = ['olsopolicy-policy-upgrade',
+                        '--policy',
+                        self.get_config_file_fullname('policy.json'),
+                        '--namespace', 'test_upgrade',
+                        '--output-file',
+                        self.get_config_file_fullname('new_policy.json'),
+                        '--format', 'json']
+            with mock.patch('sys.argv', testargs):
+                generator.upgrade_policy(conf=self.local_conf)
+                mock_log.warning.assert_any_call(policy.WARN_JSON)
+
     def test_upgrade_policy_yaml_file(self):
         test_mgr = stevedore.named.NamedExtensionManager.make_test_instance(
             extensions=self.extensions, namespace='test_upgrade')
@@ -751,3 +791,184 @@ class GetEnforcerTestCase(base.PolicyBaseTestCase):
         mock_instance.__contains__.return_value = False
         mock_manager.return_value = mock_instance
         self.assertRaises(KeyError, generator._get_enforcer, 'nonexistent')
+
+
+class ValidatorTestCase(base.PolicyBaseTestCase):
+    def _get_test_enforcer(self):
+        test_rules = [policy.RuleDefault('foo', 'foo:bar=baz'),
+                      policy.RuleDefault('bar', 'bar:foo=baz')]
+        enforcer = policy.Enforcer(self.conf)
+        enforcer.register_defaults(test_rules)
+        return enforcer
+
+    def _test_policy(self, rule, success=False, missing_file=False):
+        policy_file = self.get_config_file_fullname('test.yaml')
+        if missing_file:
+            policy_file = 'bogus.yaml'
+        self.create_config_file('test.yaml', rule)
+        self.create_config_file('test.conf',
+                                '[oslo_policy]\npolicy_file=%s' % policy_file)
+        # Reparse now that we've created our configs
+        self.conf(args=['--config-dir', self.config_dir])
+
+        with mock.patch('oslo_policy.generator._get_enforcer') as ge:
+            ge.return_value = self._get_test_enforcer()
+            result = generator._validate_policy('test')
+            if success:
+                self.assertEqual(0, result)
+            else:
+                self.assertEqual(1, result)
+
+    def test_success(self):
+        self._test_policy('foo: rule:bar', success=True)
+
+    def test_cyclical_reference(self):
+        self._test_policy('foo: rule:bar\nbar: rule:foo')
+
+    def test_invalid_syntax(self):
+        self._test_policy('foo: (bar))')
+
+    def test_false_okay(self):
+        self._test_policy('foo: !', success=True)
+
+    def test_reference_nonexistent(self):
+        self._test_policy('foo: rule:baz')
+
+    def test_nonexistent(self):
+        self._test_policy('baz: rule:foo')
+
+    def test_missing_policy_file(self):
+        self._test_policy('', missing_file=True)
+
+
+class ConvertJsonToYamlTestCase(base.PolicyBaseTestCase):
+    def setUp(self):
+        super(ConvertJsonToYamlTestCase, self).setUp()
+        policy_json_contents = jsonutils.dumps({
+            "rule1_name": "rule:admin",
+            "rule2_name": "rule:overridden",
+            "deprecated_rule1_name": "rule:admin"
+        })
+        self.create_config_file('policy.json', policy_json_contents)
+        self.output_file_path = self.get_config_file_fullname(
+            'converted_policy.yaml')
+        deprecated_policy = policy.DeprecatedRule(
+            name='deprecated_rule1_name',
+            check_str='rule:admin',
+            deprecated_reason='testing',
+            deprecated_since='ussuri',
+        )
+        self.registered_policy = [
+            policy.DocumentedRuleDefault(
+                name='rule1_name',
+                check_str='rule:admin',
+                description='test_rule1',
+                operations=[{'path': '/test', 'method': 'GET'}],
+                deprecated_rule=deprecated_policy,
+                scope_types=['system'],
+            ),
+            policy.RuleDefault(
+                name='rule2_name',
+                check_str='rule:admin',
+            )
+        ]
+        self.extensions = []
+        ext = stevedore.extension.Extension(name='test',
+                                            entry_point=None,
+                                            plugin=None,
+                                            obj=self.registered_policy)
+        self.extensions.append(ext)
+        # Just used for cli opt parsing
+        self.local_conf = cfg.ConfigOpts()
+
+        self.expected = '''# test_rule1
+# GET  /test
+# Intended scope(s): system
+#"rule1_name": "rule:admin"
+
+# rule2_name
+"rule2_name": "rule:overridden"
+
+# WARNING: Below rules are either deprecated rules
+# or extra rules in policy file, it is strongly
+# recommended to switch to new rules.
+"deprecated_rule1_name": "rule:admin"
+'''
+
+    def _is_yaml(self, data):
+        is_yaml = False
+        try:
+            jsonutils.loads(data)
+        except ValueError:
+            try:
+                yaml.safe_load(data)
+                is_yaml = True
+            except yaml.scanner.ScannerError:
+                pass
+        return is_yaml
+
+    def _test_convert_json_to_yaml_file(self, output_to_file=True):
+        test_mgr = stevedore.named.NamedExtensionManager.make_test_instance(
+            extensions=self.extensions, namespace='test')
+        converted_policy_data = None
+        with mock.patch('stevedore.named.NamedExtensionManager',
+                        return_value=test_mgr):
+            testargs = ['oslopolicy-convert-json-to-yaml',
+                        '--namespace', 'test',
+                        '--policy-file',
+                        self.get_config_file_fullname('policy.json')]
+            if output_to_file:
+                testargs.extend(['--output-file',
+                                 self.output_file_path])
+            with mock.patch('sys.argv', testargs):
+                generator.convert_policy_json_to_yaml(conf=self.local_conf)
+                if output_to_file:
+                    with open(self.output_file_path, 'r') as fh:
+                        converted_policy_data = fh.read()
+        return converted_policy_data
+
+    def test_convert_json_to_yaml_file(self):
+        converted_policy_data = self._test_convert_json_to_yaml_file()
+        self.assertTrue(self._is_yaml(converted_policy_data))
+        self.assertEqual(self.expected, converted_policy_data)
+
+    def test_convert_policy_to_stdout(self):
+        stdout = self._capture_stdout()
+        self._test_convert_json_to_yaml_file(output_to_file=False)
+        self.assertEqual(self.expected, stdout.getvalue())
+
+    def test_converted_yaml_is_loadable(self):
+        self._test_convert_json_to_yaml_file()
+        enforcer = policy.Enforcer(self.conf,
+                                   policy_file=self.output_file_path)
+        enforcer.load_rules()
+        for rule in ['rule2_name', 'deprecated_rule1_name']:
+            self.assertIn(rule, enforcer.rules)
+
+    def test_default_rules_comment_out_in_yaml_file(self):
+        converted_policy_data = self._test_convert_json_to_yaml_file()
+        commented_default_rule = '''# test_rule1
+# GET  /test
+# Intended scope(s): system
+#"rule1_name": "rule:admin"
+
+'''
+        self.assertIn(commented_default_rule, converted_policy_data)
+
+    def test_overridden_rules_uncommented_in_yaml_file(self):
+        converted_policy_data = self._test_convert_json_to_yaml_file()
+        uncommented_overridden_rule = '''# rule2_name
+"rule2_name": "rule:overridden"
+
+'''
+        self.assertIn(uncommented_overridden_rule, converted_policy_data)
+
+    def test_existing_deprecated_rules_kept_uncommented_in_yaml_file(self):
+        converted_policy_data = self._test_convert_json_to_yaml_file()
+        existing_deprecated_rule_with_warning = '''# WARNING: Below rules are either deprecated rules
+# or extra rules in policy file, it is strongly
+# recommended to switch to new rules.
+"deprecated_rule1_name": "rule:admin"
+'''
+        self.assertIn(existing_deprecated_rule_with_warning,
+                      converted_policy_data)
